#include <DPlane.h>
#include <DProfile.h>

#define NOISE_BUCKETS    1000		     /* number of histogram buckets */
#define NOISE_RANGE      0.15		     /* relative range of data to hist. */
					     /* (eg. values <= 15% of max val) */
#define NOISE_MAX_SIGMA 15.00		     /* absolute */
#define NOISE_MIN_SIGMA  5.00		     /* absolute */
#define NOISE_SIGMA      0.10		     /* 1/10th of initial estimate */


/* Global parameters and variables */

#define atanpi(x) (atan(x)/M_PI)





template<class T>
class DCanny
{
 public:
  DCanny()
    {


      CannyNoiseEstimate = 0.0;
      CannyThreshold1 = 0.0;
      CannyThreshold2 = 0.0;
    }

  _DPlane<unsigned char> do_canny(const _DPlane<T> &img, bool relative, float thresh1 = 0.0, float thresh2 = 0.0, float sigma = 1.0, int filter_size = 5);
  _DPlane<unsigned char> do_canny(const _DPlane<T> &img, float thresh1 = 0.0, float thresh2 = 0.0, float sigma = 1.0, int filter_size = 5) 
    {
      return do_canny(img, false, thresh1, thresh2, sigma, filter_size);
    }

  ~DCanny()
    {
    }

  int gradientFull(const _DPlane<float> &img, _DPlane<float> &dx, _DPlane<float> &dy, _DPlane<float> &mag);

  float cannyNoiseEstimate(const _DPlane<float> &mag);

  _DPlane<float> get_dx_plane() { return dx; }
  _DPlane<float> get_dy_plane() { return dy; }
  _DPlane<float> get_mag_plane() { return mag; }

  _DPlane<float> get_edgedir_plane();

 protected:

  inline int NMS(float *magn, const float mag, const float dx, const float dy);
  int canny_edge(const _DPlane<T> &img, const _DPlane<float> &dx, const _DPlane<float> &dy, const _DPlane<float> &magn,
		 const float thresh, _DPlane<unsigned char> &output);
  int canny_edge2(const _DPlane<T> &img, const _DPlane<float> &dx, const _DPlane<float> &dy, 
			     const _DPlane<float> &magn, const float thresh, const float thresh2,
			     _DPlane<unsigned char> &output);
  
  float CannyNoiseEstimate, CannyThreshold1, CannyThreshold2;

  long offset1[5], offset2[5];

  _DPlane<float> dx, dy, mag;
  _DPlane<unsigned char> canny_edges;


};
