#ifndef __DCLUSTER_H_
#define __DCLUSTER_H_

#include <DMatrix.h>
#include <time.h>
#include <stdlib.h>

template<class T>
struct ClusterThreadParams {
  int beg, end, actual_cluster_count;
  const _DMatrix<T> *in_matrix, *cluster_centers;
  int *assignments_ptr;
  ClusterThreadParams(int b, int e, const _DMatrix<T> *_in, const _DMatrix<T> *cc, int *assignments, int cluster_count) :
    beg(b), end(e), in_matrix(_in), cluster_centers(cc), assignments_ptr(assignments), actual_cluster_count(cluster_count) {}
};


struct ClusterThreadResult {
  ClusterThreadResult(bool _done, double _error) : done(_done), error(_error) {}
  bool done;
  double error;
};

template<class T>
class KMeansCluster
{
 public:
  KMeansCluster(int _cluster_count, int _replicate_count = 1, int _max_iters = 0, int _thread_count = 1) :
    requested_cluster_count(_cluster_count), replicate_count(_replicate_count), thread_count(_thread_count), max_iters(_max_iters)
    {
      // FIXME : put this back in at some point.
      //      srandom(time(0));
    }
  
  _DMatrix<int> do_clustering(const _DMatrix<T> &in);
  _DMatrix<int> do_clustering_singlereplicate(const _DMatrix<T> &in);

  _DMatrix<T> get_cluster_centers() const { return cluster_centers; }
  _DMatrix<int> get_assignments() const { return assignments; }
  double get_total_error() const { return total_error; }
  int get_actual_cluster_count() { return actual_cluster_count; }
  std::pair<int, T> get_assignment(const std::vector<T> &);

  friend std::ostream &operator<<(std::ostream &os, const KMeansCluster &km)
    {
      os << km.requested_cluster_count << " " << km.replicate_count << std::endl;
      os << km.actual_cluster_count << " " << km.total_error << std::endl;
      os << km.cluster_centers << std::endl;
      os << km.assignments << std::endl;

      return os;
    }

  friend std::istream &operator>>(std::istream &is, KMeansCluster &km)
    {
      is >> km.requested_cluster_count >> km.replicate_count;
      is >> km.actual_cluster_count >> km.total_error;
      is >> km.cluster_centers >> km.assignments;

      return is;
    }

 protected:
  static bool find_assignments(int beg, int end, const _DMatrix<T> &in_matrix, int *assignments_ptr, double &err, const _DMatrix<T> &cluster_centers, int actual_cluster_count);
  bool run_cluster_thread(int thread_count, int beg, int end, const _DMatrix<T> &in_matrix, int *assignments, double &total_error);
  static void *cluster_thread(void *p);

  int requested_cluster_count, replicate_count;
  int thread_count;
  int actual_cluster_count;
  int max_iters;
  double total_error;
  _DMatrix<T> cluster_centers;
  _DMatrix<int> assignments;
};



#endif
