
// simple color look-up table
class DColorLUT
{
 public:
  DColorLUT(char *filename)
    {
      FILE *fp=fopen(filename, "r");
      
      assert(fp>0);

      lut = new char[128 * 128 * 128];

      char * lut2 = new char[256];
      int index=0;
      lut2['r']=index++; lut2['g']=index++; lut2['y']=index++;
      lut2['b']=index++; lut2['o']=index++; lut2['i']=index++;
      lut2['u']=index++; lut2['w']=index++; lut2['k']=index++;
      lut2['h']=index++; lut2['a']=index++; lut2['l']=index++;
      
      for(int cp=0, r=0; r<256; r+=2)
        for(int g=0; g<256; g+=2)
	  for(int b=0; b<256; b+=2, cp++)
            {
	      int res = fgetc(fp);
	      assert(res != EOF);
	      lut[cp] = lut2[res];
            }

      delete[] lut2;
      fclose(fp);
    }

  DColorLUT()
    {
      lut = 0;
    }

  ~DColorLUT()
    {
      if(lut)
	delete[] lut;
    }

  DColorLUT &operator=(const DColorLUT &new_lut)
    {
      if(lut)
	delete lut;

      lut = new char[128 * 128 * 128];
      memcpy(lut, new_lut.lut, 128*128*128);

      return *this;
    }

  DColorLUT(const DColorLUT &new_lut)
    {
      lut = 0;
      *this = new_lut;
    }
  
  _DPlane<unsigned char> apply_to_image(const _DImage<unsigned char> &input) const
    {
      _DPlane<unsigned char> result(input.rows(), input.cols());

      assert(input.planes() == 3);

      unsigned char *r_in = input[0][0], *g_in = input[1][0], *b_in = input[2][0];
      unsigned char *out = result[0];
      int sz = input.rows() * input.cols();

      for(int i=0; i<sz; i++)
	{
	  int lut_index = ((int(r_in[i]) >> 1) << 14) + ((int(g_in[i]) >> 1) << 7) +
	    ((int(b_in[i]) >> 1));
	     
	  out[i] = lut[lut_index];
	}
	
      return result;
    }


 protected:
  char *lut;

};
