#include<DMatrix.h>
#include <complex>


// This is a very basic matrix class *without* arithmetic, but
// with some convenience functions for manipulating submatrices, setc.
//
template<class T> 
class _DComplexMatrix : public _DTwoDimArray<std::complex<T> >
{
 public:

  ///////////////////////////////////////////////////////
  // constructor
  _DComplexMatrix() : _DTwoDimArray< std::complex<T> >() {}
  _DComplexMatrix(int _rows, int _cols) : _DTwoDimArray< std::complex<T> >(_rows, _cols) {}
  _DComplexMatrix(int _rows, int _cols, const std::complex<T> *array) : _DTwoDimArray<std::complex<T> >(_rows, _cols, array) {}
  _DComplexMatrix(int _rows, int _cols, const std::complex<T> val);

  using _DTwoDimArray<std::complex<T> >::rows;
  using _DTwoDimArray<std::complex<T> >::cols;
  using _DTwoDimArray<std::complex<T> >::data_ptr;

  _DComplexMatrix<T> conj() const
    {
      _DComplexMatrix<T> result(rows(), cols());

      int sz = rows() * cols();
      std::complex<T> *result_ptr = result.data_ptr();
      const std::complex<T> *in_ptr = data_ptr();
      
      for(int i=0; i<sz; i++)
	result_ptr[i] = std::conj(in_ptr[i]);

      return result;
    }

  void operator/=(const T &v) 
    {
      int sz = rows() * cols();
      std::complex<T> *result_ptr = data_ptr();
      
      for(int i=0; i<sz; i++)
	result_ptr[i] /= v;
    }

  template<class T2>
    friend _DComplexMatrix<T2> pointwise_multiply(const _DComplexMatrix<T2> &c1, const _DComplexMatrix<T2> &c2);

  template<class T2>
    friend std::ostream &operator<<(std::ostream &is, const _DComplexMatrix<T2> &matrix);	
};


template<class T2, class T3>
static void change_type(const _DComplexMatrix<T2> &in, _DComplexMatrix<T3> &result)
{
  if(!(in.rows() == result.rows() && in.cols() == result.cols()))
    result = _DComplexMatrix<T3>(in.rows(), in.cols());
  
  int sz = in.rows() * in.cols();
  std::complex<T3> *res_ptr = result.data_ptr();
  const std::complex<T2> *in_ptr = in.data_ptr();
  
  for(int i=0; i<sz; i++)
    res_ptr[i] = std::complex<T3>(T3(in_ptr[i].real()), T3(in_ptr[i].imag()));
}



template<class T2>
static _DComplexMatrix<T2> pointwise_multiply(const _DComplexMatrix<T2> &c1, const _DComplexMatrix<T2> &c2)
{
  assert(c1.rows() == c2.rows() && c1.cols() == c2.cols());

  _DComplexMatrix<T2> result(c1.rows(), c1.cols());
  
  int sz = c1.rows() * c1.cols();
  const std::complex<T2> *c1_ptr = c1.data_ptr(), *c2_ptr = c2.data_ptr();
  std::complex<T2> *result_ptr = result.data_ptr();

  for(int i=0; i<sz; i++)
    result_ptr[i] = c1_ptr[i] * c2_ptr[i];

  return result;
}

template<class T>
static std::ostream &operator<<(std::ostream &os, const _DComplexMatrix<T> &matrix)
{
  os << matrix.rows() << " " << matrix.cols() << std::endl;

  for(int i=0; i<matrix.rows(); i++)
    {
      for(int j=0; j<matrix.cols(); j++)
	os << matrix[i][j] << " ";
      os << std::endl;
    }
  
  return os;
}



