#ifndef __DGAUSSIANMODEL_H_
#define __DGAUSSIANMODEL_H_

#include <DProfile.h>
#include <DGaussianModel.h>



template<class T>
class DGaussianMixtureModel
{
 public:

  // construct a mixture model, given the specified gaussian models and corresponding weights
  DGaussianMixtureModel(const std::vector<DGaussianModel<T> > &_models, const std::vector<float> &_weights) :
    models(_models), weights(_weights) 
    {
      assert(_models.size() == _weights.size());
      _model_likelihood = nan("");
    }

  // construct a mixture model from some training data
  DGaussianMixtureModel(const _DMatrix<T> &data, int mixture_count, float delta_thresh = 1,
			int max_iters = 50, int replicate_count = 5)
    {
      learn(data, mixture_count, delta_thresh, max_iters, replicate_count);
    }

  // learn a mixture model from some training data, given initial estimates of means and convariances
  void learn(const _DMatrix<T> &data, const std::vector<DGaussianModel<T> > &initial_models,
	     const std::vector<float> &initial_weights, float delta_thresh = 1, int max_iters = 50);

  // learn a mixture model from some training data, using k-means to initialize clusters
  void learn(const _DMatrix<T> &data, int mixture_count, float delta_thresh = 1, int max_iters = 50, int replicate_count = 5);


  // compute the likelihood of some data, given the gaussian mixture model.
  // observations are in rows, dimensions in columns.
  //
  T get_data_likelihood(const _DMatrix<T> &data);

  // compute likelihood of data given model, but also output the likelihood of each observation individually 
  //  in the likelihoods vector.
  //
  T get_data_likelihood(const _DMatrix<T> &data, std::vector<T> &likelihoods);


  DGaussianModel<T> model_of_component(int i) { return models[i]; }
  float weight_of_component(int i) { return weights[i]; }
  float peak_of_component(int i) { return weights[i] * exp(models[i].log_constant()); }

  int component_count() { return models.size(); }
  float model_likelihood() { return _model_likelihood; }

 protected:

  std::vector<DGaussianModel<T> > models;
  std::vector<float> weights;
  float _model_likelihood;
};

#endif
