//
// some miscellaneous math functions
//

//
// These are fast approximate exp() and log() functions, from
// N. Schraudolph, "A Fast, Compact Approximation of the Exponential Function,"
//   in Neural Computation, 1999.
//

/**
 * This function is on avarage 9 times faster than the system exp() function
 * and has an error of about 1.5%
 */

#ifndef __DMATH_H_
#define __DMATH_H_

#include <assert.h>
#include <float.h>

static union {
  double d;
  struct {
#if BYTE_ORDER == BIG_ENDIAN
    int i, j;
#else
    int j, i;
#endif
  } n;
} _eco;

inline double fast_exp(double val) {
  static const double a = 1048576/M_LN2;
  static const double b_c = 1072632447; /* 1072693248 - 60801 */

  if(val > 700.0)
    return FLT_MAX;
  else if(val < -700.0)
    return -FLT_MAX;

  _eco.n.i = int(a*val + b_c);

  return _eco.d;
}


static float fast_log2 (float val)
{
  assert(val > 0);
  int * const    exp_ptr = reinterpret_cast <int *> (&val);
  int            x = *exp_ptr;
  const int      log_2 = ((x >> 23) & 255) - 128;
  x &= ~(255 << 23);
  x += 127 << 23;
  *exp_ptr = x;

  val = ((-1.0f/3) * val + 2) * val - 2.0f/3;   // (1)

  return (val + log_2);
}

/**
 * This function is about 3 times faster than the system log() function
 * and has an error of about 0.01%
 */
inline float fast_log (float val)
{
  return (fast_log2 (val) * 0.69314718f);
}

inline float fast_log10 (float val)
{
  return (fast_log (val) / fast_log (10));
}

#endif
