
//
//  DLib: A simple image processing library.
//
//  David Crandall, 2003-2005
//  crandall@cs.cornell.edu
//
//  Please do not redistribute this code.
//
//
//
//
#ifndef _DPLANE_H_
#define _DPLANE_H_

#include "DMatrix.h"
#include "DRect.h"
#include <vector>

template<class T>
class _DPlane : public _DMatrix<T>
{

 public:
  _DPlane(const _DMatrix<T> &other)
    {
      _DMatrix<T>::_rows = other.rows();
      _DMatrix<T>::_cols = other.cols();
      
      _DMatrix<T>::data = 0;
      _DMatrix<T>::data_area = 0;
            
      _DMatrix<T>::initialize_storage();

      memcpy(_DMatrix<T>::data_area, other[0], 
	     _DMatrix<T>::_rows * _DMatrix<T>::_cols * sizeof(T));
    }
    
  _DPlane(int _rows, int _cols) : _DMatrix<T>(_rows, _cols)
    {
    }

  _DPlane(int _rows, int _cols, T *data) : _DMatrix<T>(_rows, _cols, data)
    {
    }
  
  _DPlane() : _DMatrix<T>() {}
  
  void draw(const DRect &rect, T color);
  void draw(const DPoint &p1, const DPoint &p2, T color);

  int total_pixel_count() const
    {
      return _DMatrix<T>::rows() * _DMatrix<T>::cols(); 
    }

  _DPlane<T> get_x_gradient() const;
  _DPlane<T> get_y_gradient() const;

  // FIXME : these are actually cross-correlation, NOT convolution!!!
  _DPlane<T> cross_correlate(const _DPlane<T> &kernel, bool normalize_flag=true) const;
  _DPlane<T> cross_correlate_fft(const _DPlane<T> &kernel, bool normalize_flag=true) const;
  _DPlane<T> cross_correlate_separable(const _DPlane<T> &row_kernel, const _DPlane<T> &col_kernel, bool normalize_flag=true) const;
  _DPlane<T> cross_correlate_1d_cols(const _DPlane<T> &kernel, bool normalize=true) const;
  _DPlane<T> cross_correlate_1d_rows(const _DPlane<T> &kernel, bool normalize=true) const;
  _DPlane<T> cross_correlate_1d_rows_sep(const _DPlane<T> &kernel, bool normalize=true) const;
  _DPlane<T> convolve_gaussian(const _DMatrix<T> &sigma, float sigma_width, bool box_approximate = false) const;
  _DPlane<T> box_filter(int kernel_rows, int kernel_cols, bool normalize) const;

  _DPlane<T> subsample(int row_factor, int col_factor);

  _DPlane<T> bilinear_interpolate(T row_offset, T col_offset) const;
  _DPlane<T> rotate_image(double angle, T bg_val=T(1e100)) const;
  _DPlane<T> rotate_image_nn(double angle) const;

  _DPlane<T> binary_thin(void) const;

  _DPlane<T> dilate(int square_size) const;
  _DPlane<T> dilate(const _DPlane<T> &kernel) const;
  _DPlane<T> dilate(const _DPlane<T> &kernel, const DPoint &se_origin) const;

  _DPlane<T> flip_horizontally() const;

  _DPlane<T> &operator=(const T &val) { _DMatrix<T>::operator=(val); return *this; }
  _DPlane<T> &operator=(const _DPlane<T> &val) { _DMatrix<T>::operator=(val); return *this; }

  void get_subsample_vector(float start_row, float row_count, int &sz, float *&ptr) const;
  _DPlane<T> bilinear_rescale(float row_factor, float col_factor) const;
  _DPlane<T> downsample_max(int scale_factor) const;

  using _DMatrix<T>::rows;
  using _DMatrix<T>::cols;


 protected:

  _DPlane<T> convolve_gaussian_approximate(T row_sigma, T col_sigma);
  _DPlane<T> box_filter_rows_1D(int kernel_cols, bool normalize) const;

};

typedef _DPlane<double> DPlane;
typedef _DPlane<short> DShortPlane;

                        



#endif
