//
//  DLib: A simple image processing library.
//
//  David Crandall, 2003-2005
//  crandall@cs.cornell.edu
//
//  Please do not redistribute this code.
//
//
//
//
#ifndef _DRECT_H_
#define _DRECT_H_

#include "DPoint.h"
#include <iostream>

template<class T>
class _DRect
{
 public:
  _DRect(const _DPoint<T> &__top_left, const _DPoint<T> &__bottom_right) : 
    _top_left(__top_left), _bottom_right(__bottom_right) {}
  _DRect(T __top, T __left, T __bottom, T __right) :
    _top_left(_DPoint<T>(__top, __left)), _bottom_right(_DPoint<T>(__bottom, __right)) {}
  _DRect()
    {
      _top_left=_DPoint<T>(-1,-1);
      _bottom_right=_DPoint<T>(-1,-1);
    }

  void top_left(const _DPoint<T> &pt) { _top_left = pt; }
  void bottom_right(const _DPoint<T> &pt) { _bottom_right = pt; }

  _DPoint<T> top_left() const { return _top_left; }
  _DPoint<T> bottom_right() const { return _bottom_right; }
  _DPoint<T> top_right() const { return _DPoint<T>(top(), right()); }
  _DPoint<T> bottom_left() const { return _DPoint<T>(bottom(), left()); }

  _DPoint<T> center_point() const 
  {
    return top_left() + size()/2;
  }

  T top() const { return _top_left.row(); }
  T bottom() const { return _bottom_right.row(); }
  T left() const { return _top_left.col(); }
  T right() const { return _bottom_right.col(); }

  void top(T a) { _top_left.row(a); }
  void bottom(T a) { _bottom_right.row(a); }
  void left(T a) { _top_left.col(a); }
  void right(T a) { _bottom_right.col(a); }

  T height() const { return bottom() - top() + 1; }
  T width() const { return right() - left() + 1; }
  _DPoint<T> size() const { return _DPoint<T>(height(), width()); }

  T area() const { return height() * width(); }

  template<class T2> friend _DRect<T2> bounding_rectangle(const _DRect<T2> &r1, const _DRect<T2> &r2);
  template<class T2> friend _DRect<T2> intersection(const _DRect<T2> &r1, const _DRect<T2> &r2);
  template<class T2> friend T2 area_of_union(const _DRect<T2> &r1, const _DRect<T2> &r2);
  template<class T2>  friend T2 area_of_intersection(const _DRect<T2> &r1, const _DRect<T2> &r2);

  bool is_within(const _DPoint<T> &pt) const
    {
      return pt.row() >= top() && pt.row() <= bottom() && pt.col() >= left() && pt.col() <= right();
    }

  //  int count_overlap(const _DRect &r2);

  _DRect<T> operator+(const _DPoint<T> &pt) const
  {
    return _DRect<T>(_top_left + pt, _bottom_right + pt);
  }

  _DRect<T> operator-(const _DPoint<T> &pt) const
  {
    return _DRect<T>(_top_left - pt, _bottom_right - pt);
  }

  _DRect<T> operator*(double factor) const
  {
    return _DRect<T>(_top_left * factor, _bottom_right * factor);
  }

  _DRect<T> operator/(double factor) const
  {
    return _DRect<T>(_top_left / factor, _bottom_right / factor);
  }

  template<class T2> friend std::ostream & operator<<(std::ostream &os, const _DRect<T2> &rect);
  template<class T2> friend std::istream & operator>>(std::istream &is, _DRect<T2> &rect);

 protected:
  _DPoint<T> _top_left, _bottom_right;
};

typedef _DRect<int> DRect;

#endif 
