//
//  DLib: A simple image processing library.
//
//  David Crandall, 2003-2005
//  crandall@cs.cornell.edu
//
//  Please do not redistribute this code.
//
//
//
//
#include <vector>
#include "DPlane.h"
#include "DImage.h"
#include "DMultiDMatrix.h"

template<class T>
class DistanceTransform_2D
{
 public:
  DistanceTransform_2D(T scaling, bool _old_algo=false) : rotate(false), angle(0), scaling_x(scaling), scaling_y(scaling), old_algo(_old_algo) {}
  DistanceTransform_2D(T _scaling_y, T _scaling_x, bool _old_algo = false) : rotate(false), angle(0), scaling_x(_scaling_x), scaling_y(_scaling_y), old_algo(_old_algo) {}
  DistanceTransform_2D(const _DMatrix<T> &sigma, bool _old_algo=false);

  _DPlane<T> do_transform(const _DPlane<T> &src_image)
    {
      return do_transform(src_image, false);
    }

  _DPlane<T> do_transform(const _DPlane<T> &src_image, _DPlane<int> &_row_locations, _DPlane<int> &_col_locations)
    {
      _DPlane<T> result = do_transform(src_image, true);

      _row_locations = row_locations;
      _col_locations = col_locations;

      return result;
    }

 protected:

  _DPlane<T> do_transform(const _DPlane<T> &src_image, bool need_locations);
  _DPlane<T> do_transform(const _DPlane<T> & in_im, bool need_locations, T scaling_x, T scaling_y);

  _DPlane<int> row_locations, col_locations;
  bool rotate;
  double angle;
  T scaling_x, scaling_y;
  bool old_algo;
};





