#include <ext/algorithm>
#include <queue>
#include <vector>
#include <set>
#include <ext/hash_map>
#include <google/sparse_hash_map>
#include "normal.h"

using namespace __gnu_cxx;
using namespace google;
using namespace std;

typedef pair<int,int> Edge;
typedef vector<TYPE> Message;
typedef vector<TYPE> Distribution;
typedef hash_map<int, Distribution> DistributionMap;
typedef hash_map<int, hash_map< int, Message > > MessageSet;
//typedef hash_map<int, hash_map< int, Message > > MessageSet;

struct PairHash : public unary_function< pair<int,int>, size_t > {
  size_t operator()(const pair<int,int> &p) const {
    return p.first * 7 + p.second;
  }
};

typedef float PD_TYPE;
class PairwiseDiff
{
public:
#ifndef TRANS
    _DMatrix<double> rot; // could change this to PD_TYPE to conserve memory, and/or switch to 
                        // a static array, but the number of pairs
                        // during rotations BP is typically not too high anyway
  //  PD_TYPE rot[9];
#endif
  //  _DMatrix<double> trans;
  PD_TYPE trans[3]; // WAS DOUBLE
  float c1_ratio, c2_ratio;
#ifndef TRANS
  float conf;
#endif

  friend istream &operator>>(istream &is, PairwiseDiff &pd)
  {
#ifndef TRANS
    pd.rot = _DMatrix<double>(3,3);
        is >> pd.rot[0][0] >> pd.rot[0][1] >> pd.rot[0][2] >> pd.rot[1][0] >> pd.rot[1][1] >> pd.rot[1][2] >> pd.rot[2][0] >> pd.rot[2][1] >> pd.rot[2][2];
	//for(int i=0; i<9; i++)
	//      is >> pd.rot[i];
#endif
    //    pd.trans = _DMatrix<double>(3,1);
    //    is >> pd.trans[0][0] >> pd.trans[0][1] >> pd.trans[0][2];
    is >> pd.trans[0] >> pd.trans[1] >> pd.trans[2];
    if(IOOptions::cratios(is))
      is >> pd.c1_ratio >> pd.c2_ratio;

    if(IOOptions::confidence(is))
      {
#ifndef TRANS
	is >> pd.conf;
#else
	float conf;
	is >> conf;
#endif
      }

    assert(is.good());

    return is;
  }

  class IOOptions
    {
    public:
      IOOptions(int opt = 0) : _options(opt) { }
    
      friend ostream &operator<<(ostream &os, const IOOptions &iof)
	{
	  os.iword(delim_index) = iof._options;
	  return os;
	}
      
      friend istream &operator>>(istream &os, const IOOptions &iof)
	{
	  os.iword(delim_index) = iof._options;
	  return os;
	}

      static bool confidence(ios_base &ios) { return !(options(ios) & NO_CONFIDENCE); }
      static bool cratios(ios_base &ios) { return !(options(ios) & NO_CRATIOS); }
      static int options(ios_base &ios) 
	{
	  return ios.iword(delim_index) ? ios.iword(delim_index) : 0;
	}
      
      typedef enum { NO_CONFIDENCE = 1, NO_CRATIOS = 2} IOOopts;

    protected:
      int _options;
      static const int delim_index;
    };
};
const int PairwiseDiff::IOOptions::delim_index = std::ios_base::xalloc();     


typedef google::sparse_hash_map<int, PairwiseDiff >  NeighborPairs;
typedef sparse_hash_map<int, NeighborPairs > Pairs;
//typedef mymap<int, NeighborPairs> Pairs;
//typedef vector< pair<int, NeighborPairs> > Pairs;
//typedef google::sparse_hash_map< int, NeighborPairs >  Pairs;

template<class T>
inline T square(T n)
{
  return n*n;
}

///////////////////////////////////////////////////////////////////////
// Distance transform stuff

// distance transform with quadratic distance
void dt(const double *src, double *dst, int s1, int s2, int d1, int d2, double scale, int off=0)
{
  int d = (d1+d2) >> 1;
  int s = s1;
  for (int p = s1; p <= s2; p++)
    if (src[s] + square(s-d-off) * scale> src[p] + square(p-d-off) * scale)
      s = p;
  dst[d] = src[s] + square(s-d-off) * scale;
      
  if(d-1 >= d1)
    dt(src, dst, s1, s, d1, d-1, scale, off);
  if(d2>=d+1)
    dt(src, dst, s, s2, d+1, d2, scale, off);
}


void dt_1d(const double *f, double scale, double *result, int beg, int end, int off=0) 
{
  dt(f, result, beg, end-1, beg, end-1, scale, off);
}


// distance transform with truncation
void dt_1d_trunc(const double *f, double scale, double *result, int beg, int end, int off=0, double trunc = 0) 
{
  //  assert(scale == 1.0);

  // find minimum value of f
  double min_f = INFINITY;
  for(int i=beg; i<end; i++)
    min_f = min(min_f, f[i]);

  min_f += trunc*trunc;

  dt(f, result, beg, end-1, beg, end-1, scale, off);

  for(int i=beg; i<end; i++)
    result[i] = min(result[i], min_f);
}

set<int> sample(const set<int> &input, int count)
{
  vector<int> result(count);
  __gnu_cxx::random_sample(input.begin(), input.end(), result.begin(), result.end());
  set<int> result2;
  result2.insert(result.begin(), result.end());
  return result2;
}

#include <sys/time.h>
int seed()
{
  struct timeval tv;
  
  gettimeofday(&tv, 0);
  
  int t = (int) ((long long)tv.tv_sec)*1000000 + tv.tv_usec;
  srand48(t);
  srand(t);

  return t;
}


// find largest connected component in graph
set<int> check_reachability(int photo_count, int edge_count, const vector<Edge> &edge_list, int &cc_ref, bool random_ref = false, bool ccref_highest_degree = false)
{
  int singletons=0;
  set<int> possibly_reachable;
  for(int ii=0; ii<edge_count; ii++)
    {
      possibly_reachable.insert(edge_list[ii].first);
      possibly_reachable.insert(edge_list[ii].second);
    }
  
  cerr << possibly_reachable.size() << " of " << photo_count << " possibly reachable" << endl;

  // compute degree of each node
  map<int, int> degrees;
  for(int ii=0; ii<edge_count; ii++)
    {
      degrees[edge_list[ii].first]++;
      degrees[edge_list[ii].second]++;
    }

  int cc_size=0;
  set<int> reachable;
  // this is a very good candidate for the World's Dumbest Connected Components algorithm. oh well, it works.
  while(possibly_reachable.size() > 1)
    {
      set<int> reachable2;
      int reference = *(possibly_reachable.begin());
      reachable2.insert(reference);
      bool done = false;

      while(!done)
	{
	  done = true;
	  for(int ii=0; ii<edge_count; ii++)
	    {
	      if(reachable2.find(edge_list[ii].first) != reachable2.end())
		{
		  if(reachable2.insert(edge_list[ii].second).second) 
		    done = false;
		}
	    }
	}

      if(ccref_highest_degree)
	{
	  int max_degree=0, max_cam=reference;
	  for(set<int>::const_iterator iter = reachable2.begin(); iter != reachable2.end(); ++iter)
	    if(max_degree < degrees[*iter])
	      max_degree = degrees[*iter], max_cam = *iter;
	  reference = max_cam;
	}

      if(random_ref)
	{
	  seed();
	  reference = *(sample(reachable2, 1).begin());
	}

      if(reachable2.size() < 25)
	singletons++;
      else
	{
	  cerr << " --- " << reference << " " << reachable2.size() << endl;
	}
      if((int)reachable2.size() > cc_size)
	cc_size = reachable2.size(), cc_ref = reference, reachable = reachable2;
      
      set<int> left;
      set_difference(possibly_reachable.begin(), possibly_reachable.end(), reachable2.begin(), reachable2.end(), inserter(left, left.begin()));
      possibly_reachable = left;
    }

  cerr << " --- and " << singletons << " components with fewer than 25 nodes " << endl;

  return reachable;
}

// find largest connected component in graph
set<int> check_reachability_new(int photo_count, const Pairs &deltas, int &cc_ref, bool random_ref = false, bool ccref_highest_degree = false)
{
  int singletons=0;
  set<int> possibly_reachable;
  map<int, int> degrees;

  // compute degree of each node
  for(Pairs::const_iterator iter=deltas.begin(); iter != deltas.end(); ++iter)
    for(NeighborPairs::const_iterator iter2=iter->second.begin(); iter2 != iter->second.end(); ++iter2)
      {
	possibly_reachable.insert(iter->first);
	possibly_reachable.insert(iter2->first);
	degrees[iter->first]++;
	degrees[iter2->first]++;
      }

  cerr << possibly_reachable.size() << " of " << photo_count << " possibly reachable" << endl;

  int cc_size=0;
  set<int> reachable;

  while(possibly_reachable.size() > 1)
    {
      set<int> reachable2;
      int reference = *(possibly_reachable.begin());
      reachable2.insert(reference);
      //     bool done = false;

      queue<int> Q;
      Q.push(reference);
      set<int> visited;

      while(!Q.empty())
	{
	  int node = Q.front();
	  Q.pop();

	  if(visited.find(node) != visited.end())
	    continue;
	  visited.insert(node);

	  Pairs::const_iterator deltas_node_iter = deltas.find(node);
	  assert(deltas_node_iter != deltas.end());
	  const NeighborPairs &adj_list = deltas_node_iter->second;
	  for(NeighborPairs::const_iterator iter=adj_list.begin(); iter != adj_list.end(); ++iter)
	    {
	      if(reachable2.find(iter->first) == reachable2.end())
		{
		  Q.push(iter->first);
		  reachable2.insert(iter->first);
		}
	    }
	}

      if(ccref_highest_degree)
	{
	  int max_degree=0, max_cam=reference;
	  for(set<int>::const_iterator iter = reachable2.begin(); iter != reachable2.end(); ++iter)
	    if(max_degree < degrees[*iter])
	      max_degree = degrees[*iter], max_cam = *iter;
	  reference = max_cam;
	}

      if(random_ref)
	{
	  seed();
	  reference = *(sample(reachable2, 1).begin());
	}

      if(reachable2.size() < 25)
      	singletons++;
      else
	cerr << " --- " << reference << " " << reachable2.size() << endl;

      if((int)reachable2.size() > cc_size)
	cc_size = reachable2.size(), cc_ref = reference, reachable = reachable2;
      
      set<int> left;
      set_difference(possibly_reachable.begin(), possibly_reachable.end(), reachable2.begin(), reachable2.end(), inserter(left, left.begin()));

      possibly_reachable = left;
    }

  cerr << " --- and " << singletons << " components with fewer than 25 nodes " << endl;

  return reachable;
}

set<int> check_reachability(int photo_count, int edge_count, const vector<Edge> &edge_list)
{
  int cc_ref=0;
  return check_reachability(photo_count, edge_count, edge_list, cc_ref);
}

set<int> check_reachability_new(int photo_count, const Pairs &deltas)
{
  int cc_ref=0;
  return check_reachability_new(photo_count, deltas, cc_ref);
}

struct BP_Params
{
  BP_Params(int _beg, int _end, const vector<Edge> *_edge_list, int _cc_ref,   
	    Pairs *_deltas, MessageSet *_last_ms, MessageSet *_current_ms, vector< _DMatrix<TYPE> > *_rotation_matrices) :
    beg_edge(_beg), end_edge(_end), edge_list(_edge_list), cc_ref(_cc_ref), deltas(_deltas), last_ms(_last_ms), current_ms(_current_ms), 
    rotation_matrices(_rotation_matrices) { }
  
  BP_Params(int _beg, int _end, const vector<Edge> *_edge_list,  Pairs *_deltas, 
	    MessageSet *_last_ms, MessageSet *_current_ms, map<int, DMatrix> *_known_locations, map<int, DMatrix> *_D_planes) :
    beg_edge(_beg), end_edge(_end), edge_list(_edge_list), deltas(_deltas), last_ms(_last_ms), current_ms(_current_ms), 
       known_locations(_known_locations), D_planes(_D_planes) { } 
                         
  int beg_edge,  end_edge;
  const vector<Edge> *edge_list;
  int  cc_ref;
  Pairs *deltas;
  MessageSet *last_ms, *current_ms;
  vector< _DMatrix<TYPE> > *rotation_matrices;
  map<int, DMatrix> *known_locations, *D_planes;
};

vector<Edge> remove_unconnected(vector<Edge> &edge_list, set<int> &reachable, Pairs &deltas)
{
  vector<Edge> edge_list2;
    Pairs deltas2;

  // FIXME : MADE THIS CHANGE
  for(int i=0; i<(int)edge_list.size(); i++)
    if(reachable.find(edge_list[i].first) != reachable.end() && reachable.find(edge_list[i].second) != reachable.end())
      {
	edge_list2.push_back(edge_list[i]);
	deltas2[edge_list[i].first][edge_list[i].second] =  deltas[edge_list[i].first][edge_list[i].second];
      }
  deltas = deltas2;
  
  return edge_list2;
}

vector<Edge> remove_unconnected_new(set<int> &reachable, Pairs &deltas)
{
  Pairs deltas2;
  int ct=0;
  for(Pairs::const_iterator iter=deltas.begin(); iter != deltas.end(); ++iter)
    for(NeighborPairs::const_iterator iter2=iter->second.begin(); iter2 != iter->second.end(); ++iter2)
      if(reachable.find(iter->first) != reachable.end() && reachable.find(iter2->first) != reachable.end())      
	ct++;

  int ii=0;
  vector<Edge> edge_list2(ct);
  deltas.set_deleted_key(-9999);

  for(Pairs::iterator iter=deltas.begin(); iter != deltas.end(); )
    {
      int v1 = iter->first;
      if(reachable.find(v1) == reachable.end())
	{
	  Pairs::iterator tmp_iter = iter;
	  ++iter;
	  deltas.erase(tmp_iter);
	}
      else
	{
	  for(NeighborPairs::iterator iter2=iter->second.begin(); iter2 != iter->second.end(); )
	    {
	      int v2=iter2->first;
	      
	      if(reachable.find(v2) == reachable.end())      
		{
		  NeighborPairs::iterator tmp_iter = iter2;
		  ++iter2;
		  iter->second.set_deleted_key(-9999);
		  iter->second.erase(tmp_iter);
		}
	      else
		{
		  edge_list2[ii] = make_pair(v1, v2);
		  ii++;
		  ++iter2;
		}
	    }

	  ++iter;
	}
    }

  return edge_list2;
}

bool check_connectivity(map<int, set<int> > neighbors, set<Edge> removed, int c1, int c2)
{
  //  cerr << "check conn" << endl;
  set<int> known, added, new_added;
  added.insert(c1);
  while(added.size() > 0)
    {
      for(set<int>::const_iterator iter = added.begin(); iter != added.end(); ++iter)
	{
	  set<int> adj_to_iter = neighbors[*iter];
	  for(set<int>::const_iterator iter2 = adj_to_iter.begin(); iter2 != adj_to_iter.end(); ++iter2)
	    if(removed.find(Edge(*iter, *iter2)) == removed.end())
	      {
		if(*iter2 == c2)
		  return true;
		else if(known.find(*iter2) == known.end())
		  new_added.insert(*iter2);
	      }
	}
      known.insert(new_added.begin(), new_added.end());
      added = new_added;
      new_added.clear();
    }

  return false;
}

// check for cut edges
// FIXME: this function doesn't handle the deltas properly. (not passed by reference)
vector<Edge> remove_cut_edges(vector<Edge> &edge_list, Pairs deltas)
{
  map<int, set<int> > neighbors;
  cerr << "building neighbors data structure..." << endl;
  for(int ii=0; ii<int(edge_list.size()); ii++)
    {
      neighbors[edge_list[ii].first].insert(edge_list[ii].second);
      neighbors[edge_list[ii].second].insert(edge_list[ii].first);
    }
  Pairs deltas2;

  vector<Edge> edge_list2;
  cerr << "checking for cut edges..." << endl;
  //  bool diagnose = false;
  for(int ii=0; ii<(int)edge_list.size(); ii++)
    {
      set<int> temp_set;
      int i = edge_list[ii].first, j = edge_list[ii].second;
      
      const set<int> &adj_i = neighbors[i], &adj_j = neighbors[j];
      set_intersection(adj_i.begin(), adj_i.end(),adj_j.begin(), adj_j.end(), inserter(temp_set, temp_set.begin()));

      if(temp_set.size() != 0)
	{
	  edge_list2.push_back(edge_list[ii]);
	  // FIXME: this is also wrong.
	  deltas2[edge_list[i].first][edge_list[i].second] =  deltas[edge_list[i].first][edge_list[i].second];
	}
    }

  deltas = deltas2;

  cerr << "removed " << edge_list.size()-edge_list2.size() << " cut edges (" << edge_list2.size() << " edges remain)"  << endl;
  return edge_list2;
}

