#include "bp-common.h"
#include "io.h"
#include <ext/hash_map>
#include <set>
#include <iomanip>
using namespace std;
using namespace __gnu_cxx;

class PackedCol
{
 public:
  PackedCol() { }
  PackedCol(double _min1, double _min2, int _min_i) : min1(_min1), min2(_min2), min_i(_min_i) { }

    // min1 is the minimum cost, min2 is the second-lowest cost, min_i is the (a) row index of min1
    float min1, min2;
    int min_i;

    friend ostream &operator<<(ostream &os, const PackedCol &pm);
    friend istream &operator>>(istream &os, PackedCol &pm);
};

ostream &operator<<(ostream &os, const PackedCol &pm)
{
  os << DIOFormat(' ') << pm.min1 << " " << pm.min2 << " " << pm.min_i;

  return os;
}

istream &operator>>(istream &is, PackedCol &pm)
{
  //  is >> pm.min1 >> pm.min2 >> pm.min_i;

  string tmp;
  is >> tmp;
  assert(is.good());
  
  if(tmp == "inf") pm.min1 = INFINITY; else pm.min1 = atof(tmp.c_str());

  is >> tmp;
  assert(is.good());
  if(tmp == "inf") pm.min2 = INFINITY; else pm.min2 = atof(tmp.c_str());

  is >> tmp;
  assert(is.good());
  pm.min_i = atoi(tmp.c_str());

  return is;
}

typedef vector< PackedCol > PackedBand;
typedef vector< PackedBand > PackedBands;
typedef vector< PackedBands > MinsSet;

class PackedMessage
{
 public:
  PackedMessage(int _rows, int _cols, MinsSet &_mins, double _samebin_penalty) : 
    mins(_mins), rows(_rows), cols(_cols), samebin_penalty(_samebin_penalty) { }
    PackedMessage() { }

    // indexed by band, then by column
    MinsSet mins;
    double angle;
    int rows, cols;
    double samebin_penalty;

    friend ostream &operator<<(ostream &os, const PackedMessage &pm);
    friend istream &operator>>(istream &os, PackedMessage &pm);
};

istream &operator>>(istream &is, PackedMessage &pm)
{
  is >> pm.angle >> pm.rows >> pm.cols >> pm.samebin_penalty;
  is >> pm.mins;

  return is;
}


ostream &operator<<(ostream &os, const PackedMessage &pm)
{
  os << DIOFormat(' ') << pm.angle << " " << pm.rows << " " << pm.cols << " " << pm.samebin_penalty << " " << pm.mins;

  return os;
}




typedef hash_map<int, hash_map< int, PackedMessage > > PackedMessageSet;
typedef float PriorsBufferType;
typedef _DMatrix<PriorsBufferType> PriorsBuffer;
typedef map<int, PriorsBuffer> PriorsMap;
/*
template<class T, class T2>
  T2 &map_lookup(map<T, T2> &m, const T &key)
{
  map<T, T2>::const_iterator iter = m.find(key);
  assert(iter != m.end());
  return iter->second;
}
*/

class UnaryPosition
{
 public:
  double confidence;
  DMatrix position;
};


template<class T1, class T2>
  struct compare_first2
  {
    bool operator()(const pair< T1, T2 > &p1, const pair< T1, T2 > &p2 ) const
    {
      return p1.first < p2.first;
    }
  };

bool compare_first(const Edge &a, const Edge &b)
{
  return (a.first < b.first) || (a.first == b.first && a.second < b.second);
}


double compare_with_gt(const hash_map<int, UnaryPosition> &gt, int photo_count, const hash_map<int, DMatrix> &estimates, const set<int> &reachable, const map<int, DMatrix> &known_locations, double &corr_err, ostream &ofs = cout)
{
  double err = 0;
  int ct=0;

  vector< pair<double,DMatrix > > offsets; 

  DMatrix offset(1,3);
  offset=0;
  for(int i=0; i<photo_count; i++)
    {
      hash_map<int, UnaryPosition>::const_iterator iter;
      if((iter = gt.find(i)) == gt.end() || estimates.find(i) == estimates.end())
	continue;
      const DMatrix &vec = estimates.find(i)->second, &correct = iter->second.position;

      double e = sqrt(pointwise_multiply(correct - vec, correct - vec).sum());

      if(reachable.find(i) != reachable.end())
	{
	  offset += (vec - correct);
	  offsets.push_back(make_pair(e, vec - correct));

	  err += e;
	  ct++;
	  
	}
    }

  const int OFFSET_METHOD = -1;
  cerr << "OFFSET METHOD" << endl;
  if(OFFSET_METHOD == 1) // use median offset
    {
      sort(offsets.begin(), offsets.end(), compare_first2<double, DMatrix>() );
      offset = offsets[offsets.size()/2].second;
    }
  else if(OFFSET_METHOD == 0) // use 0 offset
    offset = 0;
  else // use mean offset
    offset /= ct;

  offset[0][1] = 0;
  cerr << "offset " << offset << endl;
  
  corr_err=0;
  for(int i=0; i<photo_count; i++)
    {
      hash_map<int, UnaryPosition>::const_iterator iter;
      if((iter = gt.find(i)) == gt.end() || reachable.find(i) == reachable.end() || estimates.find(i) == estimates.end())
	continue;
      DMatrix vec = estimates.find(i)->second, correct = iter->second.position;
      vec[0][1] = correct[0][1] = 0;

      double e = sqrt(pointwise_multiply(correct - vec + offset, correct - vec + offset).sum());
      corr_err += e;

      ofs << "gtcost " << i << " " << e << " " << correct[0][0] << "," << correct[0][1] << "," << correct[0][2] << " " << vec[0][0] << "," << vec[0][1] << "," << vec[0][2] << endl;
    }

  corr_err /= ct;
  
  ofs << endl;

  return err/ct;
}


static pthread_mutex_t mymutex = PTHREAD_MUTEX_INITIALIZER;


class BPTrans_Inference 
{
 public:

  BPTrans_Inference() { }

  BPTrans_Inference(double _prior_trunc, double _unary_weighting, double _pair_trunc, int _bands, double _anticollapse, 
		    double _anticollapse_penalty, bool _bidir_flag, bool _precompute_priors_off, const DPoint _translate, 
		    int _GRID_COUNT2, double _DIST_GRAN2) :
    prior_trunc(_prior_trunc), unary_weighting(_unary_weighting), pair_trunc(_pair_trunc), bands(_bands), 
    anticollapse(_anticollapse), anticollapse_penalty(_anticollapse_penalty), bidir_flag(_bidir_flag), 
      precompute_priors_off(_precompute_priors_off), translate(_translate), GRID_COUNT2(_GRID_COUNT2), 
      LABEL_COUNT2(_GRID_COUNT2 * _GRID_COUNT2), DIST_GRAN2(_DIST_GRAN2) 
    {
      cerr << "label space: " << GRID_COUNT2 << " bins on single dim, " << GRID_COUNT2 * GRID_COUNT2 << " total " << endl;
      cerr << "range: " << to_dist2(0)-translate.row() << "," << to_dist2(0)-translate.col() << " " << to_dist2(GRID_COUNT2)-translate.row() << "," << to_dist2(GRID_COUNT2)-translate.col() << endl;
      cerr << "gran: " << to_dist2(1) - to_dist2(0) << endl;
    }

    // convert between various label representations
    inline int from_dist2(double s) const { return (int) round(s * double(DIST_GRAN2) + GRID_COUNT2/2); }
    inline int unpack_v0(int packed) const { return (packed / GRID_COUNT2); }
    inline int unpack_v1(int packed) const { return from_dist2(0); } // state space currently doesn't include y coordinates
    inline int unpack_v2(int packed) const { return packed % GRID_COUNT2; }
    inline int pack2(int t0, int t1, int t2) const   { return t0 * GRID_COUNT2 + t2; }
    inline double to_dist2(double s) const { return (s - GRID_COUNT2/2) / double(DIST_GRAN2); }

    // compute pairwise cost of two labels connected by edge
    //  (slow)
    double distance3_2(int my_state, int other_state, const PairwiseDiff &diff) const
    {
      DMatrix my_location(3,1);
      my_location[0][0] = to_dist2(unpack_v0(my_state)); my_location[0][1] = 0; my_location[0][2] = to_dist2(unpack_v2(my_state));
      DMatrix other_location(3,1);
      other_location[0][0] = to_dist2(unpack_v0(other_state)); other_location[0][1] = to_dist2(unpack_v1(other_state)); other_location[0][2] = to_dist2(unpack_v2(other_state));
    
      //DMatrix predicted_dir = diff.trans; 
      DMatrix predicted_dir(3,1); //,diff.trans); 
      predicted_dir[0][0] = diff.trans[0];
      predicted_dir[0][1] = diff.trans[1];
      predicted_dir[0][2] = diff.trans[2];
    
      DMatrix w(3,1);
      w = other_location - my_location;
      w = w - predicted_dir * (w.transpose() * predicted_dir)[0][0];
      double dist = ((pointwise_multiply(w, w).sum()));
    
      return dist;
    }

    PriorsBuffer compute_priors(const DMatrix &pos) const
    {
      PriorsBuffer result;
    
      DistanceTransform_2D<PriorsBufferType> dt2(1.0);
      result = PriorsBuffer(GRID_COUNT2, GRID_COUNT2);
      result = INFINITY;
      int row_bin = from_dist2(pos[0][0]);
      int col_bin = from_dist2(pos[0][2]);
      if(row_bin < 0 || row_bin >= GRID_COUNT2 || col_bin < 0 || col_bin >= GRID_COUNT2)
	cerr << "geotag outside bounds: " << pos[0][0] << " " << pos[0][2] << endl;
      else
	result[ row_bin ][ col_bin ] = 0;
      result = dt2.do_transform(result);
    
      if(prior_trunc > 0)
	pointwise_min(result, float(prior_trunc * prior_trunc));
    
      result *= unary_weighting;//*unary[iter->first].confidence;
    
      return result;
    }

    // rotate plane back
    void rotate_back(DMatrix &dt_result, DPoint target_size, double ang) const
    {
      DMatrix I4 = DPlane(dt_result).rotate_image(ang);
  
      DPoint p1 = target_size; // msg_tmp.size();
      DPoint pp1 = (I4.size() - p1) / 2;
      dt_result = I4.extract(DRect(pp1, pp1+p1-DPoint(1,1)));
      assert(dt_result.rows() == GRID_COUNT2 && dt_result.cols() == GRID_COUNT2);
    }

    // subtract off dc component of messages
    void subtract_dc(Message &this_msg) const
    {
      double sum_min_cost = 0;
      int count=0;
      for(int other_s = 0; other_s < LABEL_COUNT2; other_s++)
	if(!isinf(this_msg[other_s] ))
	  sum_min_cost += this_msg[other_s], count++;
      if(count == 0) count=1;
      sum_min_cost /= double(count);
      for(int i=0; i< LABEL_COUNT2; i++)
	this_msg[i] -= sum_min_cost;
    }

    void copy_message(Message &this_msg, const DMatrix &dt_result) const
    {
      double *dt_result_ptr = dt_result[0];
      for(int i=0, cp=0; i<GRID_COUNT2; i++)
	for(int j=0; j<GRID_COUNT2; j++, cp++)
	  if(dt_result_ptr[cp] > 1e80)
	    this_msg[cp] = INFINITY;   
	  else
	    this_msg[cp] = dt_result_ptr[cp];
    }

    void copy_message(DMatrix &msg_tmp, const vector<double> &H) const
    {
      double *msg_tmp_ptr = msg_tmp[0];
      for(int i=0, cp=0; i<GRID_COUNT2; i++)
	for(int j=0; j<GRID_COUNT2; j++, cp++)
	  msg_tmp_ptr[cp] = H[cp];
    }
  
    vector<pair<int, int> > make_band_extents(int rows) const
      {
	vector<pair<int, int> > b(bands);
      
	b[0] = make_pair(0, rows/bands);
	for(int i=1; i<bands; i++)
	  b.push_back( (make_pair(b[i-1].second-1, (i+1)*rows/bands )));

	b[bands-1].second = rows-1;

	return b;
      }

    PackedMessage make_packed_message(DMultiDMatrix &dt_result, const PairwiseDiff &this_delta) const
    {
      assert(dt_result.planes() == bands);
    
      double samebin_penalty = 0;
      if(anticollapse > 0)
	if(this_delta.c1_ratio < anticollapse && this_delta.c2_ratio < anticollapse)
	  samebin_penalty = anticollapse_penalty;

      MinsSet mins(bands, PackedBands(bands, PackedBand(dt_result.cols(), PackedCol(10e100, 10e100, 0))));

      //  cerr << sizeof(mins) << " " << sizeof(mins[0]) << " " << sizeof(mins[0][0]) << endl;

      // bidir_g == false is currently broken
      assert(bidir_flag);

      vector<pair<int, int> > band_extents = make_band_extents(dt_result.rows());

      // for each dt_transform plane
      for(int bb=0; bb < (int)mins.size(); bb++)
	{
	  DMatrix &this_dt_result = dt_result.get(bb);

	  // for each band
	  for(int b=0; b<bands; b++)
	    {
	      // for each column 
	      for(int j=0; j<dt_result.cols(); j++)
		{
		  PackedCol &_mins = mins[bb][b][j];
	  
		  if(!bidir_flag)
		    {
		      assert(false); // still broken!
		      /*	  for(int i=dt_result.rows()-1; i>=0; i--)
			{
			dt_result[i][j] = min(_mins.min1, dt_result[i][j] + samebin_penalty);
			_min = min(_min, dt_result[i][j]);
			}*/
		    }
		  else if(samebin_penalty > 0)
		    {
		      double *this_cost = this_dt_result[band_extents[b].first]+j;

		      // for each row in the band
		      for(int i=band_extents[b].first; i <= band_extents[b].second; i++, this_cost += dt_result.cols())
			{
			  if(*this_cost < _mins.min1)
			    { _mins = PackedCol(*this_cost, _mins.min1, i); }
			  else if(*this_cost < _mins.min2)
			    _mins.min2 = *this_cost;
			}
		    }
		  else
		    {
		      double *this_cost = this_dt_result[band_extents[b].first]+j;

		      // for each row in the band
		      for(int i=band_extents[b].first; i <= band_extents[b].second; i++, this_cost += dt_result.cols())
			_mins.min1 = min((double)_mins.min1, *this_cost);
		    }
		}
	    }
	}

      return PackedMessage(dt_result.rows(), dt_result.cols(), mins, samebin_penalty);
    }

    Message unpack_message(const PackedMessage &this_msg) const
    {
      DMatrix dt_result, msg_tmp;
      dt_result = DMatrix(this_msg.rows, this_msg.cols);

      vector<pair<int, int> > band_extents = make_band_extents(dt_result.rows());

      // prof.begin(6);
      for(int b=0; b < bands; b++)
	{
	  for(int j=0; j<dt_result.cols(); j++)
	    {
	      PackedCol _min(10e100, 10e100, 0);
	      //	  PackedCol _min = this_msg.mins[dt_plane][b][j];

	      for(int b2=0; b2 < bands; b2++)
		{
		  const PackedCol &this_col = this_msg.mins[ abs(b2-b) ][b][j];

		  if(_min.min1 > this_col.min1)
		    {
		      _min.min2 = min(_min.min1, this_col.min2);
		      _min.min1 = this_col.min1;
		      _min.min_i = this_col.min_i;
		    }
		  else if(_min.min2 > this_col.min1)
		    _min.min2 = this_col.min1;
		}

	      double *this_cost = dt_result[band_extents[b].first]+j;
	  
	      for(int i=band_extents[b].first; i<=band_extents[b].second; i++, this_cost += dt_result.cols())
		{
		  if(this_msg.samebin_penalty > 0 && i == _min.min_i)
		    *this_cost = min(_min.min1 + (float)this_msg.samebin_penalty, _min.min2);
		  else
		    *this_cost = _min.min1;
		}
	    }
	}
      // prof.end(6);

      // rotate back to original coordinate system
      // prof.begin(2);
      rotate_back(dt_result, DPoint(GRID_COUNT2, GRID_COUNT2), -this_msg.angle);
      // prof.end(2);
  
      // copy back to message and subtract off dc component
      // prof.begin(5);
      Message unpacked_msg(LABEL_COUNT2);
      copy_message(unpacked_msg, dt_result);
      subtract_dc(unpacked_msg);
      // prof.end(5);

      return unpacked_msg;
    }

    void make_H(vector<double> &H, int me, const map<int, DMatrix> &known_locations, const PriorsMap &D_planes) const
    {
      map<int, DMatrix>::const_iterator me_iter = known_locations.find(me);

      if(me_iter != known_locations.end())
	{
	  PriorsBuffer buf;
	  const PriorsBufferType *d_cp;

	  if(precompute_priors_off)
	    {
	      buf = compute_priors(me_iter->second);
	      d_cp = buf[0];
	    }
	  else
	    {
	      PriorsMap::const_iterator iter = D_planes.find(me);
	      assert(iter != D_planes.end());
	      d_cp = iter->second[0];
	    }

	  for(int my_s = 0; my_s < LABEL_COUNT2; my_s++)
	    H[my_s] = d_cp[my_s];
	}
      else
	{
	  for(int my_s = 0; my_s < LABEL_COUNT2; my_s++)
	    H[my_s] = 0;
	}
    }

    void read_messages(vector<double> &H, int me, int him, const Pairs &deltas, const PackedMessageSet &last_msg, int last_me, map<int, Message> &msg_cache, map< int, Message > &final) const
    {
      Pairs::const_iterator deltas_iter_me = deltas.find(me);
      assert(deltas_iter_me != deltas.end());
      const NeighborPairs &my_neighbors = deltas_iter_me->second; 
 
      PackedMessageSet::const_iterator lastmsg_me_iter = last_msg.find(me);
      assert(lastmsg_me_iter != last_msg.end());
      const hash_map<int, PackedMessage> &my_incoming_messages = lastmsg_me_iter->second;

      if(last_me != me)
	msg_cache.clear();
 
      // compute final state likelihoods (using last iteration's messages)
      //  this is done here as an optimization, since we have the unpacked messages ready
      // this is done *first*

      // FIXME: does last camera's position estimate ever get computed??
      // Answer: Yes, because it is computed *first*. However there's still a bug here because
      //  nodes whose incoming messages span multiple calls to do_iter_edge_subset won't be
      //  handled correctly.
      // Answer 2: Actually, that's not true either. A little extra work will be done if
      //  edges span the boundaries, but the results should be the same.
      // prof.begin(8);
      if(last_me != me)
	{
	  Message my_final(LABEL_COUNT2, 0); // = H;
	  for(NeighborPairs::const_iterator iter = my_neighbors.begin(); iter != my_neighbors.end(); ++iter)
	    {
	      Message unpacked_msg;
	      if(msg_cache.find(iter->first) != msg_cache.end())
		unpacked_msg = msg_cache[iter->first];
	      else
		{
		  const hash_map<int, PackedMessage>::const_iterator my_incoming_iter = my_incoming_messages.find(iter->first);
		  assert(my_incoming_iter != my_incoming_messages.end());
		  unpacked_msg = unpack_message(my_incoming_iter->second);
		  
		  if(msg_cache.size() < 100)
		    msg_cache[iter->first] = unpacked_msg;
		}

	      // FIXME: use iterators?
	      for(int my_s = 0; my_s < LABEL_COUNT2; my_s++)
		my_final[my_s] += unpacked_msg[my_s];
	    }     

	  // cache sum of all incoming messages in index -1
	  msg_cache[-1] = my_final;

	  for(int my_s = 0; my_s < LABEL_COUNT2; my_s++)
	    my_final[my_s] += H[my_s];

	  pthread_mutex_lock(&mymutex);
	  final[me] = my_final;
	  pthread_mutex_unlock(&mymutex);
	}
      // prof.end(8);

      // prof.begin(9);

      // take cached sum of all incoming messages, subtract out message coming from "him", add to H.
      // prof.begin(12);
      assert(msg_cache.find(-1) != msg_cache.end());
      if(msg_cache.find(him) == msg_cache.end())
	{
	  const hash_map<int, PackedMessage>::const_iterator my_incoming_iter = my_incoming_messages.find(him);
	  assert(my_incoming_iter != my_incoming_messages.end());
	  msg_cache[him] = unpack_message(my_incoming_iter->second);	  
	}

      Message &msg_sum = msg_cache[-1];
      Message &msg_sub = msg_cache[him];

      int my_s = 0;
      for(Message::const_iterator iter=msg_sum.begin(), iter2=msg_sub.begin(); iter != msg_sum.end(); ++iter, my_s++, ++iter2)
	H[my_s] += *iter - *iter2;
      // prof.end(12);

      // prof.end(9);

      // if msg_cache is getting full, clear it
      if(msg_cache.size() > 100)
	{
	  Message msg = msg_cache[-1];
	  msg_cache.clear();
	  msg_cache[-1] = msg;
	}
      

    }

    void do_iter_edge_subset(int beg_edge, int end_edge, const vector<Edge> &edge_list, const Pairs &deltas, 
			     const PackedMessageSet &last_ms, PackedMessageSet &current_ms, const map<int, DMatrix> &known_locations, const PriorsMap &D_planes, map< int, Message > &final, int ii) const
    {
      DMatrix msg_tmp(GRID_COUNT2, GRID_COUNT2);
      vector<double> H(LABEL_COUNT2);
  
      //FIXME bug
      vector<double> scale_factor(bands, 1);
      for(int i=1; i<bands; i++)
	scale_factor[i] = 1.0/(i * i);

      map<int, Message> msg_cache;
      int last_me = -1;
      for(int i=beg_edge; i<=end_edge; i++)
	{
	  if(!(i%100))
	    cerr << ".";
	  Edge edge = edge_list[i];
	  int me = edge.first;
	  int him = edge.second;

	  //	  const PairwiseDiff &this_delta = deltas[edge.first][edge.second];

	  Pairs::const_iterator iter1 = deltas.find(him);
	  assert(iter1 != deltas.end());
	  NeighborPairs::const_iterator iter2 = iter1->second.find(me);
	  assert(iter2 != iter1->second.end());
	  const PairwiseDiff &this_delta = iter2->second;

	  // H is my prior for this iteration, computed from my original prior and incoming messages
	  make_H(H, me, known_locations, D_planes);
      
	  if(ii != 0)
	    read_messages(H, me, him, deltas, last_ms, last_me, msg_cache, final);
      
	  copy_message(msg_tmp, H);

	  DMatrix my_predicted_dir(3,1); //,deltas[edge.second][edge.first].trans);
	  my_predicted_dir[0][0] = this_delta.trans[0];
	  my_predicted_dir[0][1] = this_delta.trans[1];
	  my_predicted_dir[0][2] = this_delta.trans[2];

	  //	  my_predicted_dir = deltas[edge.second][edge.first].trans;
      
	  // Now figure out rotation needed in x-z plane so that predicted direction is straight ahead
	  double ang = -atan2(my_predicted_dir[0][2], my_predicted_dir[0][0]) ;
      
	  // do rotation
	  // prof.begin(2);
	  DMatrix rotated =  DPlane(msg_tmp).rotate_image(ang);
	  // prof.end(2);

	  // do distance transform
	  // prof.begin(3);
	  DMultiDMatrix dt_result(3, bands, rotated.rows(), rotated.cols());
	  for(int j=0; j<bands; j++)
	    {
	      DMatrix &this_dt_result = dt_result.get(j);

	      for(int i=0; i<dt_result.rows(); i++)
		if(pair_trunc > 0)
		  dt_1d_trunc(rotated[i], scale_factor[j], this_dt_result[i], 0, this_dt_result.cols(), 0, pair_trunc);
		else
		  dt_1d(rotated[i], scale_factor[j], this_dt_result[i], 0, this_dt_result.cols(), 0);
	    }
	  // prof.end(3);

	  // min over columns and apply anti-collapse penalty
	  // prof.begin(4);
	  PackedMessage this_msg = make_packed_message(dt_result, this_delta);
	  this_msg.angle = ang;
	  
	  pthread_mutex_lock(&mymutex);
	  current_ms[him][me] = this_msg;
	  pthread_mutex_unlock(&mymutex);
	  //	  PackedMessage &this_msg = current_ms[him][me];
	  // prof.end(4);

	  last_me = me;
	}
    }

    double compute_iter_energy(const vector<int> &labels, Pairs &deltas, const vector<Edge> &edge_list) const
    {
      // FIXME : This appears to ignore the unary potentials (?)
      // compute current energy
      double cost = 0;
      for(vector<Edge>::const_iterator edge = edge_list.begin(); edge != edge_list.end(); ++edge)
	cost += distance3_2(labels[edge->first], labels[edge->second], deltas[edge->first][edge->second]);

      return cost;
    }
      
    double compute_gt_error(const vector<int> &labels, const Pairs &deltas, hash_map<int, UnaryPosition> &gt, double &corr_err, 
			    const set<int> &reachable, const map<int, DMatrix> &known_locations, ostream &ofs = cout) const
    {
      double err = 0;
      corr_err=0;

      if(gt.size() > 0)
	{
	  hash_map<int, DMatrix> estimates;
	  
	  for(int i=0; i<(int)labels.size(); i++)
	    {	    
	      int my_state = labels[i];
	      DMatrix vec(1,3);
	      vec[0][0] = to_dist2(unpack_v0(my_state)) - translate.row(); vec[0][1] = to_dist2(unpack_v1(my_state)); vec[0][2] = to_dist2(unpack_v2(my_state)) - translate.col();
	      estimates[i] = vec;
	    }
	  
	  err = compare_with_gt(gt, labels.size(), estimates, reachable, known_locations, corr_err, ofs);
	}

      return err;
    }

    int map_estimate_singlecamera(Message &final)
    {
      Message::const_iterator min_iter = min_element(final.begin(), final.end());
      return min_iter - final.begin();
    }

    vector<int> map_estimate(const map<int, Message> &final, int photo_count)
      {
	vector<int> labels(photo_count);
	map<int, Message>::const_iterator iter;
	for(int him=0; him<photo_count; him++)
	  {
	    if((iter = final.find(him)) != final.end())
	      {
		Message::const_iterator min_iter = min_element(iter->second.begin(), iter->second.end());
		labels[him] = min_iter - iter->second.begin();
	      }
	  }

	return labels;
      }


    int grid_count() const { return GRID_COUNT2; }
    int label_count() const { return LABEL_COUNT2; }
    double dist_gran() const { return DIST_GRAN2; }
    void disable_precompute_priors() { precompute_priors_off = true; }

 protected:
    double prior_trunc, unary_weighting;
    double pair_trunc;
    int bands;
    double anticollapse, anticollapse_penalty;
    bool bidir_flag;
    bool precompute_priors_off;
    DPoint translate;

    int GRID_COUNT2; // =  71; 
    int LABEL_COUNT2; // = GRID_COUNT2 * GRID_COUNT2;
    double DIST_GRAN2; // = 0.08; 

};

#define PS_STRING "ps u | grep '[b]p3-trans' 1>&2"

class BPTrans_IO
{
 public:
  static int read_pairwise_translations_file(const string &fname, vector<Edge> &edge_list, int &photo_count, Pairs &deltas)
    {
      cerr << "starting read pairwise" << endl;
      system(PS_STRING);
      int edge_count;
      ifstream ifs(fname.c_str(), ios::in);
      assert(ifs.good());
    
      edge_list.clear();
      photo_count = 0;
      edge_count = 0;

      while (ifs.good()) {
	string str;
	getline(ifs, str);
      
	if (str.size() == 0)
	  break;
	istringstream iss(str);
      
	int i, j;
	iss >> i >> j;
      
	photo_count = max(photo_count, max(i, j));
	PairwiseDiff pd;
	iss >> pd.trans[0] >> pd.trans[1] >> pd.trans[2];
	iss >> pd.c1_ratio >> pd.c2_ratio;
	deltas[i][j] = pd;
	//	edge_list.push_back(Edge(i,j));
	edge_count++;
      }

      cerr << "read pairwise" << endl;
      system(PS_STRING);

      photo_count++;
      //      edge_count = edge_list.size();
      cerr << "edge_count = " << edge_count << ",  photo_count = " << photo_count <<  endl;
      int added_ct = 0;
      Pairs deltas2;    

      for(Pairs::const_iterator iter=deltas.begin(); iter != deltas.end(); ++iter)
	for(NeighborPairs::const_iterator iter2=iter->second.begin(); iter2 != iter->second.end(); ++iter2)
	  {
	    Pairs::const_iterator iter3 = deltas.find(iter2->first);
	    if(iter3 == deltas.end() || iter3->second.find(iter->first) == iter3->second.end())
	      {
		PairwiseDiff pd;
		PD_TYPE *tr = pd.trans;
		tr[0] = -iter2->second.trans[0];
		tr[1] = -iter2->second.trans[1];
		tr[2] = -iter2->second.trans[2];

		pd.c1_ratio = iter2->second.c1_ratio;
		pd.c2_ratio = iter2->second.c2_ratio;
#ifndef TRANS
		pd.conf = iter2->second.conf;
#endif
		deltas2[iter2->first][iter->first] = pd;
		added_ct++;
		//		edge_list.push_back(Edge(iter2->first,iter->first));
	      }
	  }

      edge_count += added_ct;

      // merge deltas2 into deltas
      for(Pairs::const_iterator iter=deltas2.begin(); iter != deltas2.end(); ++iter)
	deltas[iter->first].insert(iter->second.begin(), iter->second.end());

      if(added_ct > 0)
	cerr << "WARNING: " << added_ct << " directed edges missing, adding them " << endl;

      cerr << "read " << edge_count << " edge constraints. " << endl;

      cerr << "out of read pairwise" << endl;
      system(PS_STRING);

      return edge_count;
    }

  static void read_unary_positions_file(const string &fname, hash_map<int, UnaryPosition> &unary)
    {
      ifstream ifs2(fname.c_str(), ios::in);
      assert(ifs2.good());

      while(ifs2.good())
	{
	  int cid;
	  ifs2 >> cid;
	  _DMatrix<double> rot(3,3);
	  ifs2 >> rot[0][0] >> rot[0][1] >> rot[0][2] >> rot[1][0] >> rot[1][1] >> rot[1][2] >> rot[2][0] >> rot[2][1] >> rot[2][2];
	  _DMatrix<double> trans(3,1);
	  ifs2 >> trans[0][0] >> trans[0][1] >> trans[0][2];
	  unary[cid].position = trans;
	  double conf;
	  ifs2 >> conf;
	  unary[cid].confidence = conf;
	}

      cerr << "read " << unary.size() << " unary positions. " << endl;
    }

  static hash_map<int, UnaryPosition> read_gt(const char *fname, const DPoint &translate, bool gt_trans_only)
    {
      hash_map<int, UnaryPosition> result;
      ifstream ifs(fname, ios::in);
      assert(ifs.good());
      UnaryPosition up;
      while(ifs.good())
	{
	  int camera;
	  up.position = DMatrix(1,3);

	  // supports two gt formats: either camera_id t1 t2 t3, where tx are the 3 translations,
	  //  or cid r1 r2 r3 r4 r5 r6 r7 r8 r9 t1 t2 t3
	  string str;
	  getline(ifs, str);
	  istringstream iss(str);

	  if(!ifs.good())
	    break;
	  iss >> camera;
	  if (gt_trans_only) {
	    iss >> up.position[0][0] >> up.position[0][1] >> up.position[0][2];
	  }
	  else {
	    double tmp;
	    // read the rotation matrix
	    for (int i = 0; i < 9; i++) {
	      iss >> tmp;
	    }
	    iss >> up.position[0][0] >> up.position[0][1] >> up.position[0][2];
	  }
	  up.position[0][1]=0;
	  up.confidence = INFINITY;
	  //	  up.position[0][0] += translate_row_g; up.position[0][2] += translate_col_g;
	  up.position[0][0] += translate.row(); up.position[0][2] += translate.col();
	  result[camera]= up;
	}

      cerr << "loaded ground truth for " << result.size() << " cameras " << endl;

      return result;
    }
};
  

class BPTrans_Debug
{
 public:
  BPTrans_Debug(double _stretch, const BPTrans_Inference &_bp, int _bufsz, hash_map<int, UnaryPosition> &_gt) : 
    stretch(_stretch), bp(_bp), bufsz(_bufsz), gt(_gt) { }
    
    static void draw_filledbox(_DPlane<unsigned char> &tmp, const DPoint &pt, int sz, int color, bool bg = false)
      {
	for(int j=-sz; j<=sz; j++)
	  for(int k=-sz; k<=sz; k++)
	    {
	      if(tmp.is_valid_index(DPoint(pt.col()+j, pt.row()+k)))
		{
		  if(color == -1)
		    tmp[pt.col()+j][pt.row()+k]=min(255, int(tmp[pt.col()+j][pt.row()+k])+5);
		  else
		    if(!bg || !tmp[pt.col()+j][pt.row()+k])
		      tmp[pt.col()+j][pt.row()+k] = color;
		}
	    }
      }
  
    static void draw_cross(_DPlane<unsigned char> &tmp, const DPoint &pt, int sz, int color) 
      {
	for(int j=-sz; j<=sz; j++)
	  for(int k=-sz; k<=sz; k++)
	    {
	      if(!(j == 0 || k == 0))
		continue;
	  
	      if(tmp.is_valid_index(DPoint(pt.col()+j, pt.row()+k)))
		tmp[pt.col()+j][pt.row()+k] = color;
	    }
      }
  
    void draw_grid(_DImage<unsigned char> &gt_tmp) const
    {
      for(int i=0; i< bp.grid_count(); i++)
	for(int j=0; j< bp.grid_count(); j++)
	  {
	    int x = to_imgbuf(gt_tmp[0], DPoint((int)bp.to_dist2(i), (int)bp.to_dist2(j))).row();
	    int y = to_imgbuf(gt_tmp[0], DPoint((int)bp.to_dist2(i), (int)bp.to_dist2(j))).col();
	  
	    if(x >=0 && y >= 0 && x < gt_tmp[0].rows() && y < gt_tmp[0].cols())
	      gt_tmp[0][x][y] = gt_tmp[1][x][y] = gt_tmp[2][x][y] = 255;
	  }
      //    
      //  gt_tmp[2].draw(to_imgbuf(gt_tmp[0], DPoint((int)gt[iter->first].position[0][2], (int)gt[iter->first].position[0][0])),
      //		 to_imgbuf(gt_tmp[0], DPoint((int)iter->second[0][2], (int)iter->second[0][0])), 255);
    
    }
  
    DPoint to_imgbuf(const _DPlane<unsigned char> &tmp, const DPoint &pt) const
      {
	return DPoint(int(pt.row()*stretch + tmp.rows()/2), int(pt.col()*stretch + tmp.cols()/2));
      }


    void make_gt_image(const map<int, DMatrix> &known_locations, int photo_count, const set<int> &reachable) const
    {
      _DImage<unsigned char> gt_tmp(3, bufsz, bufsz);
      if(gt.size() > 0)
	{
	  gt_tmp = 0;
	  for(int i=0; i<photo_count; i++)
	    {
	      if(gt.find(i) != gt.end() && reachable.find(i) != reachable.end())
		{
		  draw_cross(gt_tmp[2], to_imgbuf(gt_tmp[0], DPoint((int)gt[i].position[0][0], (int)gt[i].position[0][2])), 4, 255);
		  draw_number(gt_tmp[1], i, 6, DPoint(1,1)+ to_imgbuf(gt_tmp[1], DPoint((int)gt[i].position[0][2],(int)gt[i].position[0][0])));
		}
	    }
	
	  DRect rect(to_imgbuf(gt_tmp[0], DPoint((int)bp.to_dist2(0), (int)bp.to_dist2(0))), 
		     to_imgbuf(gt_tmp[0], DPoint((int)bp.to_dist2(bp.grid_count()), (int)bp.to_dist2(bp.grid_count()))));
	  gt_tmp[0].draw(rect, 128); 		
	  gt_tmp[1].draw(rect, 128); 

	  DMatrix abc(3, 1);
	  abc = 0;
	  int s=0;
	  for(map<int, DMatrix>::const_iterator iter = known_locations.begin(); iter != known_locations.end(); ++iter)
	    {
	      if(gt.find(iter->first) != gt.end())
		{
		  gt_tmp[2].draw(to_imgbuf(gt_tmp[0], DPoint((int)gt[iter->first].position[0][2], (int)gt[iter->first].position[0][0])),
				 to_imgbuf(gt_tmp[0], DPoint((int)iter->second[0][2], (int)iter->second[0][0])), 255);
		  abc[0][0] += gt[iter->first].position[0][0] - iter->second[0][0];
		  abc[0][1] += gt[iter->first].position[0][1] - iter->second[0][1];
		  abc[0][2] += gt[iter->first].position[0][2] - iter->second[0][2];
		  s++;
		}
	    }
	  cerr << "DISP " << abc / s << endl;

	  draw_grid(gt_tmp);

#ifdef HAVE_IMGIO
	  SaveDImage("gt.png", gt_tmp);
#endif 
	}
    }

    void make_iter_image(const map<int, DMatrix> &known_locations, const vector<int> &labels, int photo_count, const set<int> &reachable, int ii) const
    {
      const int sz=bufsz;
      _DImage<unsigned char> tmp(3, sz, sz);
      tmp = 0;
      for(int i=0; i<photo_count; i++)
	{
	  int my_state = labels[i];
	  DMatrix vec(3,1);
	  vec[0][0] = bp.to_dist2(bp.unpack_v0(my_state)); vec[0][1] = bp.to_dist2(bp.unpack_v1(my_state)); vec[0][2] = bp.to_dist2(bp.unpack_v2(my_state));
		    
	  if(reachable.find(i) != reachable.end())
	    {
	      draw_filledbox(tmp[0], to_imgbuf(tmp[0], DPoint((int)vec[0][0], (int)vec[0][2])), 4, -1);
		    
	      if(known_locations.find(i) != known_locations.end())
		draw_filledbox(tmp[0], to_imgbuf(tmp[0], DPoint((int)vec[0][0], (int)vec[0][2])), 1, 0);
	    }
		  
	  if(gt.size() > 0)
	    {
	      if(gt.find(i) != gt.end() && reachable.find(i) != reachable.end())
		{		  
		  draw_cross(tmp[2], to_imgbuf(tmp[0], DPoint((int)gt[i].position[0][0], (int)gt[i].position[0][2])), 4, 255);
		  if(reachable.find(i) != reachable.end())
		    tmp[1].draw(to_imgbuf(tmp[0], DPoint((int)gt[i].position[0][2], (int)gt[i].position[0][0])),
				to_imgbuf(tmp[0], DPoint((int)vec[0][2], (int)vec[0][0])), 255);
			
		  draw_number(tmp[1], i, 4, DPoint(1,1)+to_imgbuf(tmp[1], DPoint((int)gt[i].position[0][2],(int)gt[i].position[0][0])));
		}
	    }
	}
	      
      DRect rect(to_imgbuf(tmp[0], DPoint((int)bp.to_dist2(0), (int)bp.to_dist2(0))), 
		 to_imgbuf(tmp[0], DPoint((int)bp.to_dist2(bp.grid_count()), (int)bp.to_dist2(bp.grid_count()))));
      tmp[0].draw(rect, 128); 		
      tmp[1].draw(rect, 128); 
      tmp[2].draw(rect, 128); 
	    
      char temp[1024];
      sprintf(temp, "iter%02d.png", ii);
      draw_grid(tmp);

#ifdef HAVE_IMGIO
      SaveDImage(temp, tmp);
#endif
    }

 protected:
    double stretch;
    const BPTrans_Inference &bp;
    int bufsz;
    hash_map<int, UnaryPosition> &gt;
};




struct BPtrans_Params
{
  BPtrans_Params(int _beg, int _end, const vector<Edge> *_edge_list, int _cc_ref, int _ref_label,  
		 Pairs *_deltas, PackedMessageSet *_last_ms, PackedMessageSet *_current_ms, vector< _DMatrix<TYPE> > *_rotation_matrices, const BPTrans_Inference *_inf) :
    beg_edge(_beg), end_edge(_end), edge_list(_edge_list), cc_ref(_cc_ref), ref_label(_ref_label),  deltas(_deltas), last_ms(_last_ms), current_ms(_current_ms), 
    rotation_matrices(_rotation_matrices), inf(_inf) { }
  
  BPtrans_Params(int _beg, int _end, const vector<Edge> *_edge_list,  Pairs *_deltas, 
		 PackedMessageSet *_last_ms, PackedMessageSet *_current_ms, map<int, DMatrix> *_known_locations, PriorsMap *_D_planes, map< int, Message  > *_final, int _ii, const BPTrans_Inference *_inf) :
    beg_edge(_beg), end_edge(_end), edge_list(_edge_list), deltas(_deltas), last_ms(_last_ms), current_ms(_current_ms), 
    known_locations(_known_locations), D_planes(_D_planes), ii(_ii), final(_final), inf(_inf) { } 
                         
  int beg_edge,  end_edge;
  const vector<Edge> *edge_list;
  int  cc_ref, ref_label;
  Pairs *deltas;
  PackedMessageSet *last_ms, *current_ms;
  vector< _DMatrix<TYPE> > *rotation_matrices;
  map<int, DMatrix> *known_locations;
  PriorsMap *D_planes;
  int ii;
  map< int, Message > *final;
  const BPTrans_Inference *inf;
};



class BPTrans_State
{
public:
  BPTrans_State() { }

  void initialize(const char *pairwise_file, const char *unary_file, const char *gt_file, bool no_remove_cut_edges, 
	     const DPoint &translate, bool gt_trans_only, double known_from_gt, bool geo_highconf_only, const BPTrans_Inference &bp)
  {
    edge_count = BPTrans_IO::read_pairwise_translations_file(pairwise_file, edge_list, photo_count, deltas);
    //    edge_count = edge_list.size();

    //    if(!no_remove_cut_edges)
    //      edge_list = remove_cut_edges(edge_list, deltas);

    cerr << "WARNING: Cut edge removal disabled (hard-coded in bp3-trans-common.h)" << endl;

    //    edge_count = edge_list.size();

    //    reachable = check_reachability(photo_count, edge_count, edge_list);
    reachable = check_reachability_new(photo_count, deltas);

    // now remove edges that aren't involved in the connected component of interest
    edge_list = remove_unconnected_new(reachable, deltas);
    cerr << "removed " << edge_count - edge_list.size() << " irrelevant edges (" << edge_list.size() << " remaining)" << endl;

    system(PS_STRING);

    edge_count = edge_list.size();

    // sorting the edges this way makes inference faster, because decoded messages destined for the
    //  same node can be cached.
    //
    sort(edge_list.begin(), edge_list.end(), compare_first);

    // read unary data
    BPTrans_IO::read_unary_positions_file(unary_file, unary);

    if(gt_file)
      gt = BPTrans_IO::read_gt(gt_file, translate, gt_trans_only);

    if(known_from_gt > 0)
      {
	for(hash_map<int, UnaryPosition>::const_iterator iter = gt.begin(); iter != gt.end(); ++iter)
	  if(drand48() < known_from_gt && reachable.find(iter->first) != reachable.end())
	    {
	      int x = bp.from_dist2(iter->second.position[0][0]), y = bp.from_dist2(iter->second.position[0][2]);
			  
	      if(x >= 0 && x < bp.grid_count() && y >= 0 && y < bp.grid_count())
		known_locations.insert(make_pair(iter->first, iter->second.position));
	    }
      }
    else
      {
	for(hash_map<int, UnaryPosition>::iterator iter = unary.begin(); iter != unary.end(); ++iter)
	  if(fabs(unary[iter->first].position).sum() && iter->second.confidence > 0.0)
	    {
	      iter->second.position[0][0] += translate.row();
	      iter->second.position[0][2] += translate.col();
	      int x = bp.from_dist2(iter->second.position[0][0]), y = bp.from_dist2(iter->second.position[0][2]);
	      
	      if(x >= 0 && x < bp.grid_count() && y >= 0 && y < bp.grid_count() && (!geo_highconf_only || iter->second.confidence > 0.9))
		known_locations.insert(make_pair(iter->first, iter->second.position));
	    }
      }

    cerr << "# of geotags (non-uniform unary potentials) = " << known_locations.size() << endl;
    system(PS_STRING);
  }

public:
  int photo_count, edge_count;
  vector<Edge> edge_list;
  Pairs deltas;  
  set<int> reachable;
  hash_map<int, UnaryPosition> unary, gt;
  map<int, DMatrix> known_locations;

};
