#ifndef __IO_H_
#define __IO_H_

#include <ext/hash_map>

using namespace std;
using namespace __gnu_cxx;

  static void fwrite(const string &str, FILE *fp)
    {
      int tmp = str.size();
      ::fwrite(&tmp, sizeof(int), 1, fp);
      ::fwrite(str.c_str(), tmp, 1, fp);
    }

  static void fwrite(const string *str, FILE *fp)
    {
      fwrite(*str, fp);
    }

  static void fwrite(long long i, FILE *fp)
    {
      ::fwrite(&i, sizeof(long long), 1, fp);
    }

  static void fwrite(int i, FILE *fp)
    {
      ::fwrite(&i, sizeof(int), 1, fp);
    }

  static void fwrite(float i, FILE *fp)
    {
      ::fwrite(&i, sizeof(float), 1, fp);
    }

static string to_bin_string(float i)
{
  return string((char *)&i, sizeof(float));
}

static string to_bin_string(int i)
{
  return string((char *)&i, sizeof(int));
}


static void fread(string *i, FILE *fp)
{
  int sz = 0;
  fread(&sz, sizeof(int), 1, fp);

  char *str = new char[sz+1];
  fread(str, sz, 1, fp);

  if(sz > 0)
    i->assign(str, sz);
  delete[] str;
}

static void fread(string **i, FILE *fp)
{
  *i = new string;
  fread(*i, fp);
}

static void fread(long long *i, FILE *fp)
{
  fread(i, sizeof(long long), 1, fp);
}


static void fread(int *i, FILE *fp)
{
  fread(i, sizeof(int), 1, fp);
}

static void fread(float *i, FILE *fp)
{
  fread(i, sizeof(float), 1, fp);
}




template<class T>
static void fread(vector<T> &vec, FILE *fp)
{
  int sz;
  fread(&sz, sizeof(int), 1, fp);
  vec.resize(sz);
  for(int i=0; i<sz; i++)
    fread(&(vec[i]), fp);
}


template<class T>
static void fwrite(const vector<T> &vec, FILE *fp)
{
  int sz = vec.size();
  fwrite(&sz, sizeof(int), 1, fp);
  for(int i=0; i<sz; i++)
    fwrite(vec[i], fp);
}

template<class T>
static string to_bin_string(const vector<T> &vec)
{
  string result;

  result += to_bin_string((int)vec.size());
  for(int i=0; i<vec.size(); i++)
    result += to_bin_string(vec[i]);

  return result;
}



class DIOFormat
{
 public:
  DIOFormat(char c='\n') : delim_c(c) { }
    
  friend ostream &operator<<(ostream &os, const DIOFormat &iof)
    {
      os.iword(delim_index) = iof.delim_c;
      return os;
    }


  friend istream &operator>>(istream &os, const DIOFormat &iof)
    {
      os.iword(delim_index) = iof.delim_c;
      return os;
    }

  static char delim(ios_base &ios) 
    {
      return ios.iword(delim_index) ? ios.iword(delim_index) : '\n';
    }

 protected:
  int delim_c;
  static const int delim_index;
};
const int DIOFormat::delim_index = std::ios_base::xalloc();     


template<class T>
istream &operator>>(istream &is, vector<T> &vec)
{
  char tmp;
  int sz;
  is >> sz;
  //  cerr << "size " << sz << endl;
  if(DIOFormat::delim(is) != '\n')
    {
      is >> tmp;
      //      cerr << "expecting " << DIOFormat::delim(is) << " got " << tmp << endl;
      assert(tmp == DIOFormat::delim(is));
    }

  //  cerr << "reading vecotr " << sz << endl;
  vec = vector<T>(sz);
  
  for(int i=0; i<(int)vec.size(); i++)
    {
      is >> vec[i];
      if(i != ((int)vec.size() - 1))
	{
	  if(DIOFormat::delim(is) != '\n')
	    {
	      is >> tmp;
	      //      cerr << "expecting " << DIOFormat::delim(is) << " got " << tmp << endl;
	      assert(tmp == DIOFormat::delim(is));
	    }
	}
    }

  return is;
}



template<class T>
ostream &operator<<(ostream &os, const set<T> &vec)
{
  char delim = DIOFormat::delim(os);
  os << vec.size() << delim;
  
  for(typename set<T>::const_iterator iter=vec.begin(); iter != vec.end(); ++iter)
    {
      if(iter != vec.begin())
	os << delim;
      os << *iter;
    }

  return os;
}



template<class T>
ostream &operator<<(ostream &os, const vector<T> &vec)
{
  char delim = DIOFormat::delim(os);
  os << vec.size() << delim;
  
  for(typename vector<T>::const_iterator iter=vec.begin(); iter != vec.end(); ++iter)
    {
      if(iter != vec.begin())
	os << delim;
      os << *iter;
    }

  return os;
}

template<class T,class T2>
ostream &operator<<(ostream &os, const hash_map<T, T2> &vec)
{
  char delim = DIOFormat::delim(os);
  os << vec.size() << delim;
  
  for(typename hash_map<T, T2>::const_iterator iter=vec.begin(); iter != vec.end(); ++iter)
    {
      if(iter != vec.begin())
	os << delim;
      os << iter->first << ':' << iter->second;
    }

  return os;
}



/*
template<class T>
ostream &operator<<(ostream &os, const vector<T> &vec)
{
  char delim = DIOFormat::delim(os);
  os << vec.size() << delim;
  
  for(int i=0; i<(int)vec.size(); i++)
    os << vec[i] << delim;

  return os;
}
*/



template<class T>
istream &operator>>(istream &is, set<T> &vec)
{
  vector<T> tmp;
  is >> tmp;
  vec = set<T>();
  vec.insert(tmp.begin(), tmp.end());

  return is;
}

template<class T, class T2>
istream &operator>>(istream &is, hash_map<T, T2> &vec)
{
  char tmp;
  int sz;
  is >> sz;
  if(DIOFormat::delim(is) != '\n')
    {
      is >> tmp;
      assert(tmp == DIOFormat::delim(is));
    }

  vec = hash_map<T, T2>();
  
  for(int i=0; i<(int)sz; i++)
    {
      T key; T2 value; 

      is >> key >> tmp >> value;
      assert(tmp == ':');
      vec.insert(make_pair(key, value));

      if(i != ((int)vec.size() - 1))
	{
	  if(DIOFormat::delim(is) != '\n')
	    {
	      is >> tmp;
	      assert(tmp == DIOFormat::delim(is));
	    }
	}
    }

  return is;
}

#endif

